/* $Id: silver-chimera.h,v 1.1 2004/04/14 09:03:30 oohara Exp $ */

#ifndef __DANGEN_SILVER_CHIMERA_H__
#define __DANGEN_SILVER_CHIMERA_H__

#include "tenm_object.h"

tenm_object *silver_chimera_new(void);

#endif /* not __DANGEN_SILVER_CHIMERA_H__ */
