/* $Id: spellbook.h,v 1.3 2005/06/07 15:15:17 oohara Exp $ */

#ifndef __DANGEN_SPELLBOOK_H__
#define __DANGEN_SPELLBOOK_H__

#include "tenm_object.h"

tenm_object *spellbook_new(int table_index);

#endif /* not __DANGEN_SPELLBOOK_H__ */
