/* $Id: empty-wind.h,v 1.1 2003/10/25 08:52:14 oohara Exp $ */

#ifndef __DANGEN_EMPTY_WIND_H__
#define __DANGEN_EMPTY_WIND_H__

#include "tenm_object.h"

tenm_object *empty_wind_new(void);

#endif /* not __DANGEN_EMPTY_WIND_H__ */
