/* $Id: tenm_math.h,v 1.6 2002/05/10 11:01:25 oohara Exp $ */

#ifndef __TENM_MATH_H__
#define __TENM_MATH_H__
int tenm_math_init(int sqrt_size, int pow2_size);
double tenm_sin(int theta);
double tenm_cos(int theta);
double tenm_sqrt(int n);
int tenm_pow2(int n);

#endif /* not __TENM_MATH_H__ */
