/* $Id: background.h,v 1.1 2003/08/10 15:57:01 oohara Exp $ */

#ifndef __DANGEN_BACKGROUND_H__
#define __DANGEN_BACKGROUND_H__

void set_background(int n);
int clear_window_with_background(void);

#endif /* not __DANGEN_BACKGROUND_H__ */
