/* $Id: gosanpachi.h,v 1.1 2004/07/27 13:59:00 oohara Exp $ */

#ifndef __DANGEN_GOSANPACHI_H__
#define __DANGEN_GOSANPACHI_H__

#include "tenm_object.h"

tenm_object *gosanpachi_new(void);

#endif /* not __DANGEN_GOSANPACHI_H__ */
