/* $Id: tadashi.h,v 1.1 2004/05/06 13:02:13 oohara Exp $ */

#ifndef __DANGEN_TADASHI_H__
#define __DANGEN_TADASHI_H__

#include "tenm_object.h"

tenm_object *tadashi_new(void);

#endif /* not __DANGEN_TADASHI_H__ */
