/* $Id: record_util.h,v 1.4 2005/07/11 19:40:52 oohara Exp $ */

#ifndef __DANGEN_RECORD_UTIL_H__
#define __DANGEN_RECORD_UTIL_H__

int convert_to_number(int *value, const char *string);
char *concatenate_string(char *result, int n, ...);

#endif /* not __DANGEN_RECORD_UTIL_H__ */
