/* $Id: wall-13.h,v 1.1 2005/01/02 18:07:33 oohara Exp $ */

#ifndef __DANGEN_WALL_13_H__
#define __DANGEN_WALL_13_H__

#include "tenm_object.h"

tenm_object *wall_13_new(double x, double y, int t);

#endif /* not __DANGEN_WALL_13_H__ */
