/* $Id: cat-tail-grep.h,v 1.3 2004/02/26 11:31:11 oohara Exp $ */

#ifndef __DANGEN_CAT_TAIL_GREP_H__
#define __DANGEN_CAT_TAIL_GREP_H__

#include "tenm_object.h"

tenm_object *cat_tail_grep_new(void);

#endif /* not __DANGEN_CAT_TAIL_GREP_H__ */
