/* $Id: stage-clear.h,v 1.2 2004/08/18 02:49:54 oohara Exp $ */

#ifndef __DANGEN_STAGE_CLEAR_H__
#define __DANGEN_STAGE_CLEAR_H__

#include "tenm_object.h"

tenm_object *stage_clear_new(int t);

#endif /* not __DANGEN_STAGE_CLEAR_H__ */
