/* $Id: brilliance.h,v 1.4 2005/04/18 23:20:29 oohara Exp $ */

#ifndef __DANGEN_BRILLIANCE_H__
#define __DANGEN_BRILLIANCE_H__

#include "tenm_object.h"

tenm_object *brilliance_new(double x, double y, int time_stay,
                            int table_index);

#endif /* not __DANGEN_BRILLIANCE_H__ */
