/* $Id: wall-8.h,v 1.3 2004/10/25 12:09:44 oohara Exp $ */

#ifndef __DANGEN_WALL_8_H__
#define __DANGEN_WALL_8_H__

#include "tenm_object.h"

tenm_object *wall_8_new(double x, double y, int what, int t);

#endif /* not __DANGEN_WALL_8_H__ */
