/* $Id: slow.h,v 1.3 2009/11/10 18:52:56 oohara Exp $ */

#ifndef __DANGEN_SLOW_H__
#define __DANGEN_SLOW_H__

void clear_slow(void);
int do_slow(int slow_key_pressed);

#endif /* __DANGEN_SLOW_H__ */
