/* $Id: cat-tail.h,v 1.1 2004/07/17 01:50:11 oohara Exp $ */

#ifndef __DANGEN_CAT_TAIL_H__
#define __DANGEN_CAT_TAIL_H__

#include "tenm_object.h"

tenm_object *cat_tail_new(void);

#endif /* not __DANGEN_CAT_TAIL_H__ */
