/* $Id: negation-engine.h,v 1.4 2004/08/20 13:22:37 oohara Exp $ */

#ifndef __DANGEN_NEGATION_ENGINE_H__
#define __DANGEN_NEGATION_ENGINE_H__

#include "tenm_object.h"

tenm_object *negation_engine_new(void);

#endif /* not __DANGEN_NEGATION_ENGINE_H__ */
