/* $Id: pause.h,v 1.8 2004/08/16 14:13:48 oohara Exp $ */

#ifndef __DANGEN_PAUSE_H__
#define __DANGEN_PAUSE_H__

void clear_pause(void);
int do_pause(int pause_key_pressed);
void pause_by_mouse(int gain);

#endif /* not __DANGEN_PAUSE_H__ */
