/* $Id: esc-ok.h,v 1.3 2004/08/16 15:37:32 oohara Exp $ */

#ifndef __DANGEN_ESC_OK_H__
#define __DANGEN_ESC_OK_H__

void set_esc_ok(int n);
int get_esc_ok(void);

#endif /* not __DANGEN_ESC_OK_H__ */
