/* $Id: tenm_sdl_init.h,v 1.1 2002/07/09 23:28:31 oohara Exp $ */

#ifndef __TENM_SDL_INIT_H__
#define __TENM_SDL_INIT_H__

#include <SDL.h>

/* we use Uint32 here this is for internal use only */

int tenm_sdl_init(Uint32 flags);

#endif /* not __TENM_SDL_INIT_H__ */
