/* $Id: ship.h,v 1.8 2005/06/30 15:07:50 oohara Exp $ */

#ifndef __DANGEN_SHIP_H__
#define __DANGEN_SHIP_H__

void clear_ship(void);
int add_ship(int delta);
int get_ship(void);

#endif /* not __DANGEN_SHIP_H__ */
