/* $Id: plan-0.h,v 1.4 2005/07/07 11:46:55 oohara Exp $ */

#ifndef __DANGEN_PLAN_0_H__
#define __DANGEN_PLAN_0_H__

#include "tenm_object.h"

int plan_0(int t);
int plan_0_more_1_easter_egg(tenm_object *my);

#endif /* not __DANGEN_PLAN_0_H__ */
