/* $Id: seiron.h,v 1.1 2003/08/18 11:09:29 oohara Exp $ */

#ifndef __DANGEN_SEIRON_H__
#define __DANGEN_SEIRON_H__

#include "tenm_object.h"

tenm_object *seiron_new(void);

#endif /* not __DANGEN_SEIRON_H__ */
