/* $Id: p-can.h,v 1.1 2004/08/07 07:58:56 oohara Exp $ */

#ifndef __DANGEN_P_CAN_H__
#define __DANGEN_P_CAN_H__

#include "tenm_object.h"

tenm_object *p_can_new(void);

#endif /* not __DANGEN_P_CAN_H__ */
