/* $Id: warning.h,v 1.1 2004/08/10 10:28:53 oohara Exp $ */

#ifndef __DANGEN_WARNING_H__
#define __DANGEN_WARNING_H__

#include "tenm_object.h"

tenm_object *warning_new(void);

#endif /* not __DANGEN_WARNING_H__ */
