/* $Id: record_parser_private.h,v 1.3 2005/06/29 17:10:46 oohara Exp $ */

#ifndef __DANGEN_RECORD_PARSER_PRIVATE_H__
#define __DANGEN_RECORD_PARSER_PRIVATE_H__

struct _semantic_value
{
  int n0;
  int n1;
};
typedef struct _semantic_value semantic_value;

#define YYSTYPE semantic_value

/* this is generated by bison -d */
/* this must be after #define YYSTYPE */
#include "record_parser.h"

#endif /* not __DANGEN_RECORD_PARSER_PRIVATE_H__ */
