/* $Id: stage-select.h,v 1.25 2005/06/30 18:44:58 oohara Exp $ */

#ifndef __DANGEN_STAGE_SELECT_H__
#define __DANGEN_STAGE_SELECT_H__

#include "record_data.h"

int stage_select(game_record *record);

#endif /* not __DANGEN_STAGE_SELECT_H__ */
