/* $Id: mankanshoku.h,v 1.1 2004/11/04 11:47:47 oohara Exp $ */

#ifndef __DANGEN_MANKANSHOKU_H__
#define __DANGEN_MANKANSHOKU_H__

#include "tenm_object.h"

tenm_object *mankanshoku_new(void);

#endif /* not __DANGEN_MANKANSHOKU_H__ */
