/* $Id: net-can-howl.h,v 1.11 2003/09/24 02:52:08 oohara Exp $ */

#ifndef __DANGEN_NET_CAN_HOWL_H__
#define __DANGEN_NET_CAN_HOWL_H__

#include "tenm_object.h"

tenm_object *net_can_howl_core_new(void);

#endif /* not  __DANGEN_NET_CAN_HOWL_H__ */
