/* $Id: last-boss.h,v 1.1 2003/07/13 14:53:15 oohara Exp $ */

#ifndef __PHOTON_B_LAST_BOSS_H__
#define __PHOTON_B_LAST_BOSS_H__

#include "tenm_object.h"

tenm_object *last_boss_new(void);

#endif /* not __PHOTON_B_LAST_BOSS_H__ */
