/* $Id: seiron-fake.h,v 1.1 2003/08/22 17:09:45 oohara Exp $ */

#ifndef __DANGEN_SEIRON_FAKE_H__
#define __DANGEN_SEIRON_FAKE_H__

#include "tenm_object.h"

tenm_object *seiron_fake_new(void);

#endif /* not __DANGEN_SEIRON_FAKE_H__ */
