/* $Id: afterdeath.h,v 1.1 2004/07/19 13:23:46 oohara Exp $ */

#ifndef __DANGEN_AFTERDEATH_H__
#define __DANGEN_AFTERDEATH_H__

#include "tenm_object.h"

tenm_object *afterdeath_new(void);

#endif /* not __DANGEN_AFTERDEATH_H__ */
