/* $Id: nexus.h,v 1.5 2004/12/23 13:32:08 oohara Exp $ */

#ifndef __DANGEN_NEXUS_H__
#define __DANGEN_NEXUS_H__

#include "tenm_object.h"

tenm_object *nexus_new(void);

#endif /* not __DANGEN_NEXUS_H__ */
