/* $Id: stage-title.h,v 1.1 2004/08/17 14:53:12 oohara Exp $ */

#ifndef __DANGEN_STAGE_TITLE_H__
#define __DANGEN_STAGE_TITLE_H__

#include "tenm_object.h"

tenm_object *stage_title_new(void);

#endif /* not __DANGEN_STAGE_TITLE_H__ */
