/* $Id: strikers.h,v 1.1 2004/07/08 13:25:23 oohara Exp $ */

#ifndef __DANGEN_STRIKERS_H__
#define __DANGEN_STRIKERS_H__

#include "tenm_object.h"

tenm_object *strikers_new(void);

#endif /* not __DANGEN_STRIKERS_H__ */
