/* $Id: theorem-weapon.h,v 1.1 2004/04/25 09:34:22 oohara Exp $ */

#ifndef __DANGEN_THEOREM_WEAPON_H__
#define __DANGEN_THEOREM_WEAPON_H__

#include "tenm_object.h"

tenm_object *theorem_weapon_new(void);

#endif /* not __DANGEN_THEOREM_WEAPON_H__ */
