/* $Id: wall-11.h,v 1.2 2004/12/12 16:09:06 oohara Exp $ */

#ifndef __DANGEN_WALL_11_H__
#define __DANGEN_WALL_11_H__

#include "tenm_object.h"

tenm_object *wall_11_new(double x, double y, double speed, int theta,
                         int t_shoot);

#endif /* not __DANGEN_WALL_11_H__ */
