/* $Id: watcher-below.h,v 1.1 2004/07/14 14:59:08 oohara Exp $ */

#ifndef __DANGEN_WATCHER_BELOW_H__
#define __DANGEN_WATCHER_BELOW_H__

#include "tenm_object.h"

tenm_object *watcher_below_new(void);

#endif /* not __DANGEN_WATCHER_BELOW_H__ */
