/* $Id: insane-hand.h,v 1.1 2004/06/08 06:22:08 oohara Exp $ */

#ifndef __DANGEN_INSANE_HAND_H__
#define __DANGEN_INSANE_HAND_H__

#include "tenm_object.h"

tenm_object *insane_hand_new(void);

#endif /* not __DANGEN_INSANE_HAND_H__ */
