/* $Id: respiration.h,v 1.2 2004/09/06 18:05:00 oohara Exp $ */

#ifndef __DANGEN_RESPIRATION_H__
#define __DANGEN_RESPIRATION_H__

#include "tenm_object.h"

tenm_object *respiration_new(int table_index);

#endif /* not __DANGEN_RESPIRATION_H__ */
