/* $Id: hugin.h,v 1.1 2004/07/23 12:47:28 oohara Exp $ */

#ifndef __DANGEN_HUGIN_H__
#define __DANGEN_HUGIN_H__

#include "tenm_object.h"

tenm_object *hugin_new(void);

#endif /* not __DANGEN_HUGIN_H__ */
