/* $Id: wall-4.h,v 1.1 2005/06/19 00:53:23 oohara Exp $ */

#ifndef __DANGEN_WALL_4_H__
#define __DANGEN_WALL_4_H__

#include "tenm_object.h"

tenm_object *wall_4_new(double x, double y, double speed_slide, 
                        int t_slide, int t_slide_initial);

#endif /* not __DANGEN_WALL_4_H__ */
