/* $Id: hatsuda.h,v 1.1 2004/04/21 06:28:57 oohara Exp $ */

#ifndef __DANGEN_HATSUDA_H__
#define __DANGEN_HATSUDA_H__

#include "tenm_object.h"

tenm_object *hatsuda_new(void);

#endif /* not __DANGEN_SILVER_CHIMERA_H__ */
