/* $Id: perpeki.h,v 1.4 2003/10/17 14:37:39 oohara Exp $ */

#ifndef __DANGEN_PERPEKI_H__
#define __DANGEN_PERPEKI_H__

#include "tenm_object.h"

tenm_object *perpeki_new(void);

#endif /* not __DANGEN_PERPEKI_H__ */
