/* $Id: w-ko.h,v 1.1 2004/05/12 22:50:43 oohara Exp $ */

#ifndef __DANGEN_W_KO_H__
#define __DANGEN_W_KO_H__

#include "tenm_object.h"

tenm_object *w_ko_new(void);

#endif /* not __DANGEN_W_KO_H__ */
